<?php
/**
 * PurchasedFromCategories class for checking if a contact has purchased a specific product or not.
 *
 * @author [WPFunnels Team]
 * @email [support@rextheme.com]
 * @create date 2023-05-15 11:03:17
 * @modify date 2023-05-15 11:03:17
 * @package /app
 */

namespace MintMail\App\Internal\Automation\Condition;

/**
 * PurchasedFromCategories class for checking 
 * if a contact has purchased a specific product or not.
 * 
 * @since 1.2.7
 */
class PurchasedFromCategories {

	/**
	 * Checks if any segment ID from the given segments array matches any group ID in the automation data.
	 *
	 * @param array $segments The array of segments to check for matching group IDs.
	 * @param array $automation_data The automation data containing the relevant information.
	 *
	 * @return bool Returns true if a segment ID matches a group ID, false otherwise.
	 * @since 1.2.7
	 */
	public function included_in( array $value, array $automation_data ): bool {
		$order_id = isset( $automation_data['data']['order_id'] ) ? $automation_data['data']['order_id'] : '';

		if ( empty( $order_id ) ) {
			return false;
		}

		$order = wc_get_order($order_id);
		$items = $order->get_items();
		$current_category_ids = [];

        foreach ($items as $item) {
			$product = $item->get_product();
		
			if ($product && $product->get_id()) {
				// Get the category IDs associated with the product
				$product_category_ids = wp_get_post_terms($product->get_id(), 'product_cat', ['fields' => 'ids']);
				$current_category_ids = array_merge($current_category_ids, $product_category_ids); // Add to the category IDs array
			}
		}
		
		// Remove duplicate category IDs
		$current_category_ids = array_unique($current_category_ids);
		$category_ids         = array_values( array_column( $value, 'value' ) );
		// Find the intersection between group IDs and segment IDs.
		$intersect = array_intersect($current_category_ids, $category_ids);
		// If there is at least one common ID, return true.
		if ( !empty( $intersect ) ) {
			return true;
		}

		return false;
	}

	/**
	 * Checks if none of the segment IDs from the given segments array are present in the group IDs of the automation data.
	 *
	 * @param array $segments           The value to check if it does not exist within the first name.
	 * @param array $automation_data The automation data containing the first name.
	 *
	 * @return bool                   True if the value does not exist within the first name, false otherwise.
	 * @since 1.2.7
	 */
	public function not_included_in( array $value, array $automation_data ): bool {
		$order_id = isset( $automation_data['data']['order_id'] ) ? $automation_data['data']['order_id'] : '';

		if ( empty( $order_id ) ) {
			return false;
		}

		$order = wc_get_order($order_id);
		$items = $order->get_items();
		$current_category_ids = [];

        foreach ($items as $item) {
			$product = $item->get_product();
		
			if ($product && $product->get_id()) {
				// Get the category IDs associated with the product
				$product_category_ids = wp_get_post_terms($product->get_id(), 'product_cat', ['fields' => 'ids']);
				$current_category_ids = array_merge($current_category_ids, $product_category_ids); // Add to the category IDs array
			}
		}
		
		// Remove duplicate category IDs
		$current_category_ids = array_unique($current_category_ids);
		$category_ids         = array_values( array_column( $value, 'value' ) );

		// Find the segment IDs that are not present in the group IDs.
		$diff = array_diff( $current_category_ids, $category_ids );

		if ( !empty( $diff ) ) {
			return true;
		}

		return false;
	}
}
