<?php

/**
 * WooCommerce merge tag parser
 *
 * @author [WPFunnels Team]
 * @email [support@getwpfunnels.com]
 * @create date 2024-09-13 09:03:17
 * @modify date 2024-03-03 09:03:17
 * @package 
 */

namespace MailMintPro\App\Internal\EmailCustomization\Parser;

use WC_Subscription;

/**
 * Class WCMergeTagParser
 *
 * This class is responsible for parsing WooCommerce merge tags.
 *
 * @since 1.15.4
 */
class WCMergeTagParser
{

    /**
     * Value key
     *
     * @var string
     * @since 1.15.4
     */
    protected $value_key;

    /**
     * Default value
     *
     * @var string
     * @since 1.15.4
     */
    protected $default_value;

    /**
     * Params
     *
     * @var array
     * @since 1.15.4
     */
    protected $params;

    /**
     * Subscription ID
     *
     * @var int
     * @since 1.15.4
     */
    protected $subscription_id;

    /**
     * Constructor for the WCMergeTagParser class.
     *
     * @param string $value_key The key for the value to be parsed.
     * @param mixed $default_value The default value to return if the key is not found.
     * @param array $params Additional parameters, including 'subscription_id'.
     * 
     * @since 1.15.4
     */
    public function __construct($value_key, $default_value, $params)
    {
        $this->value_key = $value_key;
        $this->default_value = $default_value;
        $this->params = $params;
        $this->subscription_id = isset($params['subscription_id']) ? $params['subscription_id'] : 0;
    }

    /**
     * Parses the WC Subscriptions merge tag based on the provided value key.
     *
     * @return mixed The parsed value based on the value key,
     * or the default value if the key is not found.
     * 
     * @since 1.15.4
     */
    public function parse_wc_subscription_merge_tag(){
        $subscription =  wcs_get_subscription($this->subscription_id);

        if (! $subscription instanceof WC_Subscription) {
            return $this->default_value;
        }

        // Parse the merge tag based on the value key.
        switch ($this->value_key) {
            case 'id':
                return $this->subscription_id ? $this->subscription_id : $this->default_value;
            case 'status':
                $subscription_status_slug = 'wc-' . $subscription->get_status();
                $subscription_statuses    = wcs_get_subscription_statuses();
                $subscription_status      = $subscription_statuses[$subscription_status_slug];
                return $subscription_status ? $subscription_status : $this->default_value;
            case 'product_name':
                $items = $subscription->get_items();
                $product_names = [];
                foreach ($items as $item) {
                    $product_names[] = $item->get_name();
                }
                return !empty($product_names) ? implode(', ', $product_names) : $this->default_value;
            case 'product_price':
                $items = $subscription->get_items();
                $product_prices = [];
                foreach ($items as $item) {
                    $product_prices[] = $item->get_total();
                }
                return !empty($product_prices) ? implode(', ', $product_prices) : $this->default_value;
            case 'start_date':
                return $subscription->get_date_created() ? date_i18n(get_option('date_format'), strtotime($subscription->get_date_created())) : $this->default_value;
            case 'end_date':
                return $subscription->get_date( 'end', 'site' ) ? date_i18n(get_option('date_format'), strtotime($subscription->get_date( 'end', 'site' ))) : $this->default_value;
            case 'trial_end_date':
                return $subscription->get_date('trial_end') ? date_i18n(get_option('date_format'), strtotime($subscription->get_date('trial_end'))) : $this->default_value;
            case 'last_payment_date':
                return $subscription->get_date('last_order_date_created') ? date_i18n(get_option('date_format'), strtotime($subscription->get_date('last_order_date_created'))) : $this->default_value;
            case 'next_payment_date':
                return $subscription->get_date('next_payment', 'site') ? date_i18n(get_option('date_format'), strtotime($subscription->get_date('next_payment', 'site'))) : $this->default_value;
            case 'view_url':
                return $subscription->get_view_order_url() ? $subscription->get_view_order_url() : $this->default_value;
            case 'billing_company':
                return $subscription->get_billing_company() ? $subscription->get_billing_company() : $this->default_value;
            case 'billing_address':
                return $subscription->get_formatted_billing_address() ? $subscription->get_formatted_billing_address() : $this->default_value;
            case 'payment_method':
                return $subscription->get_payment_method_to_display() ? $subscription->get_payment_method_to_display() : $this->default_value;
            case 'total':
                return $subscription->get_total() ? wc_price($subscription->get_total()) : $this->default_value;
            case 'order_id':
                return $subscription->get_parent_id() ? $subscription->get_parent_id() : $this->default_value;
            default:
                return $this->default_value;
        }
    }
}
